﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace JednorozmernePoleWPF
{
    /// <summary>
    /// Interakční logika pro MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Datové složky třídy (globální proměnné)
        // Specifikace konstanty a inicializace
        const int Max = 10;
        // Aktuální rozměr pole - počet prvků
        private int pocetPrvku;
        // Přípustná minimální hodnota (dolní mez) prvku - inicializace
        private int minHodnota = 1;
        // Přípustná maximální hodnota (horní mez) prvku - inicializace
        private int maxHodnota = 100;

        // Deklarace pole
        private int[] pole;
        #endregion Datové složky třídy
        public MainWindow()
        {
            InitializeComponent();

            /* Přiřazení maximálního rozměru vektoru do datové složky určující
             * počet prvků vektoru */
            this.pocetPrvku = Max;

            // Vytvoření instance (objektu) pole s definovanou délkou
            this.pole = new int[this.pocetPrvku];

            /* Přiřazení počtu prvků vektoru do objektu na formuláři určujícího
             * velikost vektoru*/
            this.TxtPocetPrvku.Text = this.pocetPrvku.ToString();

            /* Přiřazení přípustné minimální hodnoty prvku ve vektoru do objektu 
             * na formuláři určujícího mezní hodnotu */
            this.TxtMinHodnota.Text = this.minHodnota.ToString();

            /* přiřazení přípustné maximální hodnoty prvku ve vektoru do objektu (pole) 
             * na formuláři určujícího mezní hodnotu */
            this.TxtMaxHodnota.Text = this.maxHodnota.ToString();
        }

        private void BtnKonec_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        #region Validace

        /// <summary>
        /// validace rozměru vektoru
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TxtPocetPrvku_LostFocus(object sender, RoutedEventArgs e)
        {
            //lokální proměnná pro uložení nového počtu prvků
            int novaHodnota;
            int.TryParse(this.TxtPocetPrvku.Text, out novaHodnota);

            // pokud počet prvků je větší než max. rozměr vektoru, nebo počet prvků je menší než 1
            if ((novaHodnota > Max) || (novaHodnota < 1))
            {
                MessageBox.Show(String.Format("Chyba velikosti vektoru, nastavení na max. rozměr: {0}", Max));
                novaHodnota = Max;
                this.TxtPocetPrvku.Text = Max.ToString();
            }

            //pokud nový rozměr vektoru není stejný jako stávající počet prvků vektoru
            if (novaHodnota != this.pocetPrvku)
            {
                this.pocetPrvku = novaHodnota;
                this.pole = new int[this.pocetPrvku];
                this.ZobrazitPoleVListView(this.pole);
                this.ZobrazitPoleJakoText(novaHodnota, this.pole);
            }
        }

        /// <summary>
        /// validace dolní meze intervalu připustných hodnot prvku
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TxtMinHodnota_LostFocus(object sender, RoutedEventArgs e)
        {
            int.TryParse(this.TxtMinHodnota.Text, out this.minHodnota);

            if (this.minHodnota > this.maxHodnota)
            {
                MessageBox.Show(String.Format("chyba hodnoty dolní meze - dolní mez nastavena na horní mez: {0}", this.maxHodnota));
                this.minHodnota = this.maxHodnota;
                this.TxtMinHodnota.Text = minHodnota.ToString();
            }


        }

        private void TxtMaxHodnota_LostFocus(object sender, RoutedEventArgs e)
        {
            int.TryParse(this.TxtMaxHodnota.Text, out this.maxHodnota);

            if (this.maxHodnota < this.minHodnota)
            {
                MessageBox.Show(String.Format("chyba hodnoty dolní meze - dolní mez nastavena na horní mez: {0}", this.maxHodnota));
                this.maxHodnota = this.minHodnota;
                this.TxtMaxHodnota.Text = maxHodnota.ToString();
            }

        }

        /// <summary>
        /// Validace indexu prvku vektoru při ručním plnění
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TxtPrvekIndex_LostFocus(object sender, RoutedEventArgs e)
        {
            // lokální proměnnou pro index prvku
            int index;
            int.TryParse(this.TxtPrvekIndex.Text, out index);

            if ((index > this.pocetPrvku - 1) || (index < 0))
            {
                MessageBox.Show(String.Format("Chybný index prvku. Index nastaven na horní mez: {0}", this.pocetPrvku - 1));
                this.TxtPrvekIndex.Text = $"{this.pocetPrvku - 1}";
            }
        }

        /// <summary>
        /// Validace hodnoty prvku vektoru při ručním zadání
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TxtHodnota_LostFocus(object sender, RoutedEventArgs e)
        {
            //lokální proměnná t - hodnota prvku
            int t;
            int.TryParse(this.TxtHodnota.Text, out t);

            if ((t > this.maxHodnota) || (t < this.minHodnota))
            {
                MessageBox.Show($"Hodnota prvku není v intervalu: {this.minHodnota} až {this.maxHodnota}");
                this.TxtHodnota.Clear();
            }

        }

        #endregion

        #region PlneniPoli

        /// <summary>
        /// Metoda pro plnění prvků vektoru náhodnými čísly
        /// </summary>
        /// <param name="n"> počet prvků pole</param>
        /// <param name="minh">minimální celočíselná hodnota prvku</param>
        /// <param name="maxh">maximální celočíselná hodnota prvku</param>
        /// <param name="pole">jednorozměrné pole</param>
        private void NahodnePole(int n, int minh, int maxh, int[] pole)
        {
            Random rnd = new Random();

            for (int i = 0; i < n; i++)
            {
                pole[i] = rnd.Next(minh, maxh + 1);
            }
        }



       
        /// <summary>
        /// Volání metody pro plnění vektoru náhodnými čísly
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnNahodne_Click(object sender, RoutedEventArgs e)
        {
            this.NahodnePole(this.pocetPrvku, this.minHodnota, this.maxHodnota, this.pole);
        }

        /// <summary>
        /// Metoda pro nulování vektoru
        /// </summary>
        /// <param name="n"> počet prvků vektoru </param>
        /// <param name="pole"> jednorozměrné pole</param>
        private void NulovatPole(int n, int[] pole)
        {
            for (int i = 0; i < n; i++)
            {
                pole[i] = 0;
            }
        }



        private void BtnNulovat_Click(object sender, RoutedEventArgs e)
        {
            this.NulovatPole(this.pocetPrvku, this.pole);
        }

        // TODO: udělat ještě Metodu pro ruční plnění

        #endregion
    }
}
